/*
  Copyright (C) 2007 Bradley Arsenault

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "LANGameInformation.h"
#include "Stream.h"

LANGameInformation::LANGameInformation(const YOGGameInfo& information)
{

}



LANGameInformation::LANGameInformation()
{

}



void LANGameInformation::encodeData(GAGCore::OutputStream* stream) const
{
	stream->writeEnterSection("LANGameInformation");
	gameInfo.encodeData(stream);
	stream->writeLeaveSection();
}



void LANGameInformation::decodeData(GAGCore::InputStream* stream)
{
	stream->readEnterSection("LANGameInformation");
	gameInfo.decodeData(stream);
	stream->readLeaveSection();
}



const YOGGameInfo& LANGameInformation::getGameInformation() const
{
	return gameInfo;
}



YOGGameInfo& LANGameInformation::getGameInformation()
{
	return gameInfo;
}


